package orderservicelogic

import (
	"app-cloudep-order-server/gen_result/pb/order"
	model "app-cloudep-order-server/internal/model/mongo"
	"context"

	ers "code.30cm.net/digimon/library-go/errs"
	"github.com/shopspring/decimal"

	"app-cloudep-order-server/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListOrderLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewListOrderLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListOrderLogic {
	return &ListOrderLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

type GetOrderListReq struct {
	PageIndex int64 `json:"page_index" validate:"required"`
	PageSize  int64 `json:"page_size" validate:"required"`

	ReferenceID   string  `json:"reference_id"`
	ReferenceUID  string  `json:"reference_uid"`
	BusinessID    string  `json:"business_id"`
	UID           string  `json:"uid"`
	OrderType     int     `json:"order_type"`
	DirectionType []int64 `json:"direction_type"`
	OrderStatus   []int64 `json:"order_status"`

	StartCreateTime       int64 `json:"start_create_time"`
	EndCreateTime         int64 `json:"end_create_time"`
	StartUpdateTime       int64 `json:"start_update_time"`
	EndUpdateTime         int64 `json:"end_update_time"`
	StartOrderArrivalTime int64 `json:"start_order_arrival_time"`
	EndOrderArrivalTime   int64 `json:"end_order_arrival_time"`
	StartOrderPaymentTime int64 `json:"start_order_payment_time"`
	EndOrderPaymentTime   int64 `json:"end_order_payment_time"`

	CryptoType string `json:"crypto_type"`
	TxHash     string `json:"tx_hash"`
}

// ListOrder 取得訂單列表
func (l *ListOrderLogic) ListOrder(in *order.ListOrderReq) (*order.ListOrderResp, error) {
	// 驗證資料，目前只有 Page 必帶，其他要驗證在驗證
	if err := l.svcCtx.Validate.ValidateAll(&GetOrderListReq{
		PageIndex: in.GetPageIndex(),
		PageSize:  in.GetPageSize(),
	}); err != nil {
		// 錯誤代碼 06-011-00
		return nil, ers.InvalidFormat(err.Error())
	}

	orders, total, err := l.svcCtx.OrderModel.ListOrder(l.ctx, model.GetOrderListReq{})
	if err != nil {
		return nil, err
	}

	res := make([]*order.GetOrderResp, 0, len(orders))
	for _, item := range orders {
		res = append(res, ConvertOrderToGetOrderResp(item))
	}

	return &order.ListOrderResp{
		Data: res,
		Page: &order.Pager{
			Total: total,
			Index: in.GetPageIndex(),
			Size:  in.GetPageSize(),
		},
	}, nil
}

// ConvertOrderToGetOrderResp 將 Order 結構轉換為 GetOrderResp
func ConvertOrderToGetOrderResp(o model.Order) *order.GetOrderResp {
	return &order.GetOrderResp{
		BusinessId:           o.BusinessID,
		OrderType:            int32(o.OrderType),
		OrderStatus:          int32(o.OrderStatus),
		Brand:                o.Brand,
		OrderUid:             o.OrderUID,
		ReferenceId:          o.ReferenceID,
		Count:                o.Count.String(),
		OrderFee:             o.OrderFee.String(),
		Amount:               o.Amount.String(),
		ReferenceBrand:       optionalString(o.ReferenceBrand),
		ReferenceUid:         optionalString(o.ReferenceUID),
		WalletStatus:         optionalInt64(o.WalletStatus),
		ThreePartyStatus:     optionalInt64(o.ThreePartyStatus),
		DirectionType:        optionalInt64(o.DirectionType),
		CryptoType:           optionalString(o.CryptoType),
		ThirdPartyFee:        optionalDecimalToString(o.ThirdPartyFee),
		CryptoToUsdtRate:     optionalDecimalToString(o.CryptoToUSDTRate),
		FiatToUsdRate:        optionalDecimalToString(o.FiatToUSDRate),
		FeeCryptoToUsdtRate:  optionalDecimalToString(o.FeeCryptoToUSDTRate),
		UsdtToCryptoTypeRate: optionalDecimalToString(o.USDTToCryptoTypeRate),
		PaymentFiat:          optionalString(o.PaymentFiat),
		PaymentUnitPrice:     optionalDecimalToString(o.PaymentUnitPrice),
		PaymentTemplateId:    optionalString(o.PaymentTemplateID),
		OrderArrivalTime:     optionalInt64(o.OrderArrivalTime),
		OrderPaymentTime:     optionalInt64(o.OrderPaymentTime),
		UnpaidTimeoutSecond:  optionalInt64(o.UnpaidTimeoutSecond),
		ChainType:            optionalString(o.ChainType),
		TxHash:               optionalString(o.TxHash),
		FromAddress:          optionalString(o.FromAddress),
		ToAddress:            optionalString(o.ToAddress),
		ChainFee:             optionalDecimalToString(o.ChainFee),
		ChainFeeCrypto:       optionalString(o.ChainFeeCrypto),
		Memo:                 optionalString(o.Memo),
		OrderNote:            optionalString(o.OrderNote),
		CreateTime:           o.CreateTime,
		UpdateTime:           o.UpdateTime,
	}
}

// Helper functions for optional fields
func optionalString(s *string) *string {
	if s != nil {
		return s
	}

	return nil
}

func optionalInt64(i *int64) *int64 {
	if i != nil {
		return i
	}

	return nil
}

func optionalDecimalToString(d *decimal.Decimal) *string {
	if d != nil {
		s := d.String()

		return &s
	}

	return nil
}

func ConvertOrdersToGetOrderResp(orders []model.Order) []*order.GetOrderResp {
	res := make([]*order.GetOrderResp, 0, len(orders))
	for _, o := range orders {
		res = append(res, ConvertOrderToGetOrderResp(o))
	}

	return res
}
